@@ -82,18 +82,10 @@ public final class HTMLManagerServlet extends ManagerServlet {
     static final String connectorTrustedCertsJspPath = "/WEB-INF/jsp/connectorTrustedCerts.jsp";
 
     private boolean showProxySessions = false;
+    private String htmlSubTitle = null;
 
     // --------------------------------------------------------- Public Methods
 
-    /**
-     * Process a GET request for the specified resource.
-     *
-     * @param request  The servlet request we are processing
-     * @param response The servlet response we are creating
-     *
-     * @exception IOException      if an input/output error occurs
-     * @exception ServletException if a servlet-specified error occurs
-     */
     @Override
     public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
 
@@ -144,15 +136,6 @@ public final class HTMLManagerServlet extends ManagerServlet {
         list(request, response, message, smClient);
     }
 
-    /**
-     * Process a POST request for the specified resource.
-     *
-     * @param request  The servlet request we are processing
-     * @param response The servlet response we are creating
-     *
-     * @exception IOException      if an input/output error occurs
-     * @exception ServletException if a servlet-specified error occurs
-     */
     @Override
     public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
 
@@ -321,6 +304,9 @@ public final class HTMLManagerServlet extends ManagerServlet {
         Object[] args = new Object[2];
         args[0] = getServletContext().getContextPath();
         args[1] = smClient.getString("htmlManagerServlet.title");
+        if (htmlSubTitle != null) {
+            args[1] += "</font><br/><font size=\"+1\">" + htmlSubTitle;
+        }
 
         // HTML Header Section
         writer.print(MessageFormat.format(Constants.HTML_HEADER_SECTION, args));
@@ -742,17 +728,11 @@ public final class HTMLManagerServlet extends ManagerServlet {
     }
 
 
-    /**
-     * @see jakarta.servlet.Servlet#getServletInfo()
-     */
     @Override
     public String getServletInfo() {
-        return "HTMLManagerServlet, Copyright (c) 1999-2023, The Apache Software Foundation";
+        return "HTMLManagerServlet, Copyright (c) 1999-2024, The Apache Software Foundation";
     }
 
-    /**
-     * @see jakarta.servlet.GenericServlet#init()
-     */
     @Override
     public void init() throws ServletException {
         super.init();
@@ -761,6 +741,8 @@ public final class HTMLManagerServlet extends ManagerServlet {
         String value = null;
         value = getServletConfig().getInitParameter("showProxySessions");
         showProxySessions = Boolean.parseBoolean(value);
+
+        htmlSubTitle = getServletConfig().getInitParameter("htmlSubTitle");
     }
 
     // ------------------------------------------------ Sessions administration
